/*
	Fiero Zombie controller board.
	Copyright Dennis Clark and TTT Enterprises 2005
	All rights reserved, you are free to use this code
	as long as you tell everyone where you got it.
*/

//********************************************************************
// The dirty configuration details
//********************************************************************
#include <18F252.h>
#include "zombie.h"
//#device ICD=TRUE
#fuses H4,NOWDT,NOPROTECT,NOBROWNOUT,NOLVP
#use delay(clock=40000000)
#use rs232(baud=9600, xmit=PIN_C6, rcv=PIN_C7)	//hardware UART
#use FAST_IO(C)
#use FAST_IO(B)
#use FAST_IO(A)

//***********************************************
// ISR save locations for low priority interrupts
//***********************************************
int8	W_save;
#locate W_save=0x7D
int8	STATUS_save;
#locate STATUS_save=0x7E
int8	BSR_save;
#locate BSR_save=0x7F

//*********************
// Globals and statics
//*********************
unsigned int		ChanA=114;					//preset to center position (off)
unsigned int		ChanB=114;
unsigned int		lastChanB=114;
unsigned int		lastChanA=114;
//unsigned int	ChanC=114;
//unsigned int	lastChanC=114;

unsigned char	RxDataIn;						//the last byte received by UART
unsigned char 	TxDataOut;						//the next byte to be transmitted

int		chanState=0;							//State in the ISR routine

unsigned int16	t_52ms=0;						//52ms counter worked in the ISR

//********************************************************************
//Functions
//********************************************************************

void init(void)
//Initialize the processor and set up the I/O registers
{
	/*
	RA0	MODE LED		output
	RA1 UNIT LED		output
	RA2 Chan A R/C		input
	RA3	Chan B R/C		input
	RA4	Remote/Auto		input
	RA5	Set Button		input
	*/
	set_tris_a(0b11111100);
	
	/*
	RB0	Light A			output
	RB1	Light B			output
	RB2	Light C			output
	RB3	Light D			output
	RB4	Relay			output
	RB5	Audio en		output
	RB6	PGC				input
	RB7	PGD				input
	*/
	set_tris_b(0b11000000);
	
	/*
	RC0	Chan C R/C	input
	RC1	PWM2 Mot 3 & 4	output
	RC2	PWM1 Mot 1 & 2	output
	RC3	Dir2 Mot 3 & 4	output
	RC4	Dir1 Mot 1 & 2	output
	RC5	SPKR			output
	RC6	TxD (UART)		output
	RC7	RxD (UART)		input
	*/
	set_tris_c(0b10000000);
	
	setup_adc_ports(NO_ANALOGS);						//All lines are now digital
	setup_adc(ADC_OFF);
	output_B(0x00);										//turn everything off
	output_C(0x00);
	
	//Set up useable timers
	setup_timer_0(RTCC_8_BIT | RTCC_DIV_128);			//To time the servo pulses

	RCONbits.IPEN = 1;									//inable priority IRQs
	
	setup_timer_1(T1_INTERNAL);							//for 10us interrupt
	enable_interrupts(INT_TIMER1);
	IPR1bits.TMR1IP = 1;								//Make it a high priority interrupt
	set_timer1(S_10us);
	
	//set up the PWM
//	setup_timer_2(T2_DIV_BY_16,0xFF,1);					//Frequency= 2.44KHz for cheap motors
	setup_timer_2(T2_DIV_BY_4,0x7F,1);					//Frequency= 19KHz for my Escap motors
	setup_ccp1(CCP_PWM);
	setup_ccp2(CCP_PWM);
	set_pwm1_duty(0);									//turn off the PWM signals
	set_pwm2_duty(0);
	
	//finally, set up the background timer
	setup_timer_3(T3_INTERNAL | T3_DIV_BY_8);
	set_timer3(0);
	
	//Setup the USART for interrupts
	PIE1bits.RCIE = 1;									//turn on Rx interrupts
	IPR1bits.RCIP = 0;									//low priority
	PIR1bits.RCIF = 0;									//clear it now
	
	enable_interrupts(GLOBAL);
}

void beep(int hPeriod, int16 sDuration)
{
	int16 y;	
	for (y=0;y<sDuration;y++)
	{
		output_bit(SPKR,1);
		delay_us(hPeriod);
		output_bit(SPKR,0);
		delay_us(hPeriod);
	}
}

void MotorA(int chanA)
/*
	Deals with setting PWM for PWM1 
*/
{
	signed int speed;
	unsigned int pwm;
	unsigned int chan;
	
	chan = chanA;
	
	if (chan > 154)							//Limit my endpoints to be the same for both sticks
		chan = 154;
	else if (chan < 74)
		chan = 74;

	if (chan == lastChanA)
		return;
		
	speed = 114 - chan;						// positive for forward, negative for reverse
	
	if (chan > 112 && chan < 116)			// create our deadband
	{
		pwm=0;
	}
	else
	{
		if ((lastChanA > 114 && chan < 114) || (lastChanA < 114 && chan > 114))
		{
			setup_ccp1(CCP_OFF);			//Here we coast a bit rather than suddenly
			delay_ms(50);					//change motor directions to save wear and
		}									//tear on the gear train!
		if (speed < 0)
			output_bit(DIR_1,1);
		else
			output_bit(DIR_1,0);
//		pwm = (abs(speed)*5)+25;				//For 2.44KHz 10% to 90%
		pwm = (abs(speed)*3);					//19KHz 5% to 95%
	}
	setup_ccp1(CCP_PWM);
	set_pwm1_duty(pwm);
	lastChanA = chan;
}

void MotorB(int chanB)
/*
	Deals with setting PWM for PWM2
*/
{
	signed int speed2;
	unsigned int pwm2;
	unsigned int chan;
	
	chan = chanB;
	
	if (chan > 154)
		chan = 154;
	else if (chan < 74)
		chan = 74;
		
	if (chan == lastChanB)
		return;
		
	speed2 = 114 - chan;
	
	if (chan > 112 && chan < 116)
	{
		pwm2=0;
	}
	else
	{
		if ((lastChanB > 114 && chan < 114) || (lastChanB < 114 && chan > 114))
		{
			setup_ccp2(CCP_OFF);				//stop current before changing directions
			delay_ms(50);
		}
			
		if (speed2 < 0)
			output_bit(DIR_2,1);
		else
			output_bit(DIR_2,0);
//		pwm2 = (abs(speed2)*5)+25;					//for 2.44KHz 10% to 90%
		pwm2 = (abs(speed2)*3);						//19KHz 5% to 95%
	}
	setup_ccp2(CCP_PWM);
	set_pwm2_duty(pwm2);
	lastChanB = chanB;
}

void HeartBeat(void)
{
	if (t_52ms == 2)
	{
		MODE_LED_OFF;
		UNIT_LED_ON;
	}
	else if (t_52ms >= 4)
	{
		MODE_LED_ON;
		UNIT_LED_OFF;
		t_52ms = 0;
	}			
}

//********************************************************************
// The main event
//********************************************************************

void main(void)
{
	
	init();
	printf("%c%c",16,12);
	delay_ms(10);
	printf ("hello\n\r");
	delay_ms(1000);
	
	t_52ms = 0;
	MODE_LED_ON;
	UNIT_LED_OFF;
	
	disable_interrupts(GLOBAL);

	printf("IPR1bits = %x \n\r",IPR1);
	printf("PIE1bits = %x \n\r",PIE1);
	printf("PIR1bits = %x \n\r",PIR1);

	enable_interrupts(GLOBAL);
	
	while (1 == 1)
	{
		
		MotorA(chanA);
		MotorB(chanB);
		HeartBeat();
		
	}
}


//*********************
// ISR's
//*********************
void ISR_High()
/*
 The generic high priority fast interrupt, first we have to set registers to 
 get high priority, CCS does not handle this well. (See Init code).
 This operates as a state machine that will ONLY look at one edge of one signal each
 Time the interrupt occurs.  This keeps things moving along fast, and since each
 Interrupt is 10us later, it takes a total of 40us to read both channels.
 The essential registers are automatically saved in an 18F high priority IRQ.
*/
{
	switch (chanState)
	{
	case SET_C1:								//Channel A count
		if (input(PIN_A2) == 1)
		{
			TMR0L = 0;
			chanState++;
		}
		break;
	case READ_C1:
		if (input(PIN_A2) == 0)
		{
			ChanA = TMR0L;
			chanState++;
		}
		break;
	case SET_C2:								//Channel B count
		if (input(PIN_A3) == 1)
		{
			TMR0L = 0;
			chanState++;
		}
		break;
	case READ_C2:
		if (input(PIN_A3) == 0)
		{
			ChanB = TMR0L;;
			chanState = SET_C1;;
		}
		break;
	}

	PIR1bits.TMR1IF = 0;							//clear interrupt bit
	set_timer1(S_10us);								//reset timer clock
	
	
	if(PIR2bits.TMR3IF)								//This is a simple 52ms clock
	{
		t_52ms++;									//inc counter and clear the timer
		TMR3L=0;
		TMR3H=0;
		PIR2bits.TMR3IF=0;
	}
	
	#asm
	retfie 1
	#endasm
}

void ISR_Low(void)
{
	//First save those essential bits, they aren't safe here from a high priority IRQ.
	INTCONbits.GIEL = 0;							//disable low priority ISRs for now
	#asm
	movwf W_save
	#endasm
	STATUS_save = STATUS;
	BSR_save = BSR;
	
	if (PIR1bits.RCIF)							//Check for USART in IRQ
	{
		RxDataIn = RCREG;						//Get received data
		while(~TXSTAbits.TRMT);					//wait for it to finish
		TXREG = RxDataIn;						//auto echo
	}	
	
	//Now get those essential bits back, we're done.
	BSR = BSR_save;
	STATUS = STATUS_save;
	#asm
	movf W_save,W
	#endasm
	INTCONbits.GIEL = 1;							//enable low priority ISRs for now
	#asm
	retfie 0
	#endasm
}
/*
 *I'm handling the interrupts myself rather than use CCS interrupt routines.  This will be much
 *faster and leaner.
 */
#org 0x0008,0x000A
void ISR_High_Wrapper(void)
{
	ISR_High();
}
#org 0x0018,0x001A
void ISR_Low_Wrapper(void)
{
	ISR_Low();
}
